@@ -1,11 +1,10 @@
 import typing
 
 import click
-import httpx
 import typer
 
-from neosctl import constant
 from neosctl import schema
+from neosctl import util
 from neosctl.util import check_profile_exists
 from neosctl.util import get_user_profile
 from neosctl.util import is_success_response
@@ -43,20 +42,19 @@ def ensure_login(method):
 
         check_profile_exists(ctx)
 
-        if ctx.obj.profile.auth_flow == constant.AuthFlow.keycloak.value:
-            # Try to refresh token
-            # Confirm it is a token invalid 401, registry not configured mistriggers this flow.
-            if r.status_code == 401:
-                data = r.json()
-                if "code" in data and data["code"].startswith("A0"):
-                    refresh_token(ctx)
+        # Try to refresh token
+        # Confirm it is a token invalid 401, registry not configured mistriggers this flow.
+        if r.status_code == 401:
+            data = r.json()
+            if "code" in data and data["code"].startswith("A0"):
+                refresh_token(ctx)
 
-                    # Refresh the context
-                    c = read_config_dotfile()
-                    ctx.obj.config = c
-                    ctx.obj.profile = get_user_profile(c, ctx.obj.profile_name)
+                # Refresh the context
+                c = read_config_dotfile()
+                ctx.obj.config = c
+                ctx.obj.profile = get_user_profile(c, ctx.obj.profile_name)
 
-                    r = method(*args, **kwargs)
+                r = method(*args, **kwargs)
 
         return r
     return check_access_token
@@ -73,7 +71,7 @@ def _update_profile(
         user=ctx.obj.profile.user,
         access_token=auth.access_token,
         refresh_token=auth.refresh_token,
-        auth_flow=ctx.obj.profile.auth_flow,
+        ignore_tls=ctx.obj.profile.ignore_tls,
     )
 
     return profile
@@ -84,6 +82,8 @@ def login(
     ctx: typer.Context,
     password: typing.Optional[str] = typer.Option(None, "--password", "-p"),
 ):
+    """Login to neos.
+    """
     check_profile_exists(ctx)
 
     if password is None:
@@ -95,7 +95,8 @@ def login(
             hide_input=True,
         )
 
-    r = httpx.post(
+    r = util.post(
+        ctx,
         "{auth_url}/login".format(auth_url=auth_url(ctx.obj.get_gateway_api_url())),
         json={"user": ctx.obj.profile.user, "password": password},
     )
@@ -113,11 +114,14 @@ def login(
 
 @app.command()
 def logout(ctx: typer.Context):
+    """Logout from neos.
+    """
     check_profile_exists(ctx)
 
     _check_refresh_token_exists(ctx)
 
-    r = httpx.post(
+    r = util.post(
+        ctx,
         "{auth_url}/logout".format(auth_url=auth_url(ctx.obj.get_gateway_api_url())),
         json={"refresh_token": ctx.obj.profile.refresh_token},
     )
@@ -137,7 +141,8 @@ def refresh_token(ctx: typer.Context):
     check_profile_exists(ctx)
     _check_refresh_token_exists(ctx)
 
-    r = httpx.post(
+    r = util.post(
+        ctx,
         "{auth_url}/refresh".format(auth_url=auth_url(ctx.obj.get_gateway_api_url())),
         json={"refresh_token": ctx.obj.profile.refresh_token},
     )
