@@ -69,11 +69,11 @@ def common(
 
 
 app = typer.Typer(name="neosctl", callback=callback)
-app.add_typer(profile.app, name="profile")
-app.add_typer(auth.app, name="auth", callback=common)
-app.add_typer(consume.app, name="consume", callback=common)
-app.add_typer(metadata.app, name="metadata", callback=common)
-app.add_typer(product.app, name="product", callback=common)
-app.add_typer(registry.app, name="registry", callback=common)
-app.add_typer(spark.app, name="spark", callback=common)
-app.add_typer(iam.app, name="iam", callback=common)
+app.add_typer(profile.app, name="profile", help="Manage profiles.")
+app.add_typer(auth.app, name="auth", callback=common, help="Manage authentication status.")
+app.add_typer(consume.app, name="consume", callback=common, help="Consume published data products.")
+app.add_typer(metadata.app, name="metadata", callback=common, help="Manage and browse metadata.")
+app.add_typer(product.app, name="product", callback=common, help="Manage data products.")
+app.add_typer(registry.app, name="registry", callback=common, help="Manage cores and search data products.")
+app.add_typer(spark.app, name="spark", callback=common, help="Manage data product spark/secrets.")
+app.add_typer(iam.app, name="iam", callback=common, help="Manage access policies.")
