@@ -8,8 +8,6 @@ from typing import Optional
 
 from pydantic import BaseModel
 
-from neosctl import constant
-
 
 class FieldDataType(BaseModel):
     meta: Dict[str, Any]
@@ -51,12 +49,9 @@ class OptionalProfile(BaseModel):
     registry_api_url: str = ""
     iam_api_url: str = ""
     user: str = ""
-    auth_flow: constant.AuthFlow = constant.AuthFlow.keycloak
     access_token: str = ""
     refresh_token: str = ""
-
-    class Config:
-        use_enum_values = True
+    ignore_tls: bool = False
 
 
 class Profile(BaseModel):
@@ -64,12 +59,9 @@ class Profile(BaseModel):
     registry_api_url: str
     iam_api_url: str
     user: str
-    auth_flow: constant.AuthFlow
     access_token: str
     refresh_token: str
-
-    class Config:
-        use_enum_values = True
+    ignore_tls: bool
 
 
 class EffectEnum(Enum):
