@@ -4,15 +4,15 @@ import typing
 import httpx
 import typer
 
+from neosctl import util
 from neosctl.auth import ensure_login
-from neosctl.util import bearer
 from neosctl.util import process_response
 from neosctl.util import send_output
 
 
 app = typer.Typer()
 secret_app = typer.Typer()
-app.add_typer(secret_app, name="secret")
+app.add_typer(secret_app, name="secret", help="Manage secrets for a spark job.")
 
 
 def spark_url(name: str, gateway_api_url: str) -> str:
@@ -37,21 +37,15 @@ def add_job(
     driver_memory: str = typer.Option("512m", "--driver-memory", "-dm"),
     spark_version: str = typer.Option("3.1.1", "--spark-version", "-sv"),
 ):
+    """Assign a spark job.
+
+    Assign and configure a spark job for a data product. This will result in a
+    one off run of the spark job.
+    """
     @ensure_login
-    def _request(
-        ctx: typer.Context,
-        product_name: str,
-        f: typing.IO,
-        docker_tag: str,
-        executor_cores: int,
-        executor_instances: int,
-        executor_memory: str,
-        driver_cores: int,
-        driver_core_limit: str,
-        driver_memory: str,
-        spark_version: str,
-    ) -> httpx.Response:
-        return httpx.post(
+    def _request(ctx: typer.Context, f: typing.IO) -> httpx.Response:
+        return util.post(
+            ctx,
             spark_url(product_name, ctx.obj.gateway_api_url),
             params={
                 "docker_tag": docker_tag,
@@ -64,7 +58,6 @@ def add_job(
                 "spark_version": spark_version,
             },
             files={"python_file": f},
-            headers=bearer(ctx),
         )
 
     fp = pathlib.Path(job_filepath)
@@ -75,32 +68,22 @@ def add_job(
         )
 
     with fp.open("rb") as f:
-        r = _request(
-            ctx,
-            product_name,
-            f,
-            docker_tag,
-            executor_cores,
-            executor_instances,
-            executor_memory,
-            driver_cores,
-            driver_core_limit,
-            driver_memory,
-            spark_version,
-        )
+        r = _request(ctx, f)
 
     process_response(r)
 
 
 @app.command()
 def job_status(ctx: typer.Context, product_name: str):
+    """Get current status of a spark job.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.get(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.get(
+            ctx,
             "{spark_url}".format(spark_url=spark_url(product_name, ctx.obj.gateway_api_url)),
-            headers=bearer(ctx),
         )
-    r = _request(ctx, product_name)
+    r = _request(ctx)
 
     process_response(r)
 
@@ -111,13 +94,15 @@ def render_logs(payload: typing.Dict):
 
 @app.command()
 def job_logs(ctx: typer.Context, product_name: str):
+    """Get logs for a spark job.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.get(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.get(
+            ctx,
             "{spark_url}/log".format(spark_url=spark_url(product_name, ctx.obj.gateway_api_url)),
-            headers=bearer(ctx),
         )
-    r = _request(ctx, product_name)
+    r = _request(ctx)
 
     process_response(r, render_logs)
 
@@ -136,19 +121,14 @@ def update_job(
     driver_memory: str = typer.Option(None, "--driver-memory", "-dm"),
     spark_version: str = typer.Option(None, "--spark-version", "-sv"),
 ):
+    """Update an assigned spark job.
+
+    Update the assigned spark job file and/or the spark job configuration values.
+    """
     @ensure_login
     def _request(
         ctx: typer.Context,
-        product_name: str,
         f: typing.Optional[typing.IO],
-        docker_tag: typing.Optional[str],
-        executor_cores: typing.Optional[int],
-        executor_instances: typing.Optional[int],
-        executor_memory: typing.Optional[str],
-        driver_cores: typing.Optional[int],
-        driver_core_limit: typing.Optional[str],
-        driver_memory: typing.Optional[str],
-        spark_version: typing.Optional[str],
     ) -> httpx.Response:
         params = {
             k: v
@@ -166,11 +146,11 @@ def update_job(
         }
         files = {"python_file": f} if f else {}
 
-        return httpx.put(
+        return util.put(
+            ctx,
             spark_url(product_name, ctx.obj.gateway_api_url),
             params=params,
             files=files,
-            headers=bearer(ctx),
         )
 
     if job_filepath:
@@ -180,48 +160,44 @@ def update_job(
                 msg="Can not find file: {}".format(fp),
                 exit_code=1,
             )
-    args = [
-        docker_tag,
-        executor_cores,
-        executor_instances,
-        executor_memory,
-        driver_cores,
-        driver_core_limit,
-        driver_memory,
-        spark_version,
-    ]
     if job_filepath:
         with fp.open("rb") as f:
-            r = _request(ctx, product_name, f, *args)
+            r = _request(ctx, f)
     else:
-        r = _request(ctx, product_name, None, *args)
+        r = _request(ctx, None)
 
     process_response(r)
 
 
 @app.command()
 def remove_job(ctx: typer.Context, product_name: str):
+    """Remove assigned spark job.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.delete(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.delete(
+            ctx,
             spark_url(product_name, ctx.obj.gateway_api_url),
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name)
+    r = _request(ctx)
     process_response(r)
 
 
 @app.command()
 def trigger_job(ctx: typer.Context, product_name: str):
+    """Trigger assigned spark job.
+
+    Trigger an additional run of a spark job.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.post(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.post(
+            ctx,
             "{spark_url}/trigger".format(spark_url=spark_url(product_name, ctx.obj.gateway_api_url)),
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name)
+    r = _request(ctx)
     process_response(r)
 
 
@@ -231,17 +207,21 @@ def schedule_job(
     product_name: str = typer.Option(..., "--product-name", "-pn"),
     schedule: str = typer.Option(..., "--schedule", "-s", help='Schedule in crontab format (e.g. "* * * * *")'),
 ):
+    """Schedule an assigned spark job.
+
+    Schedule a spark job once it is configured correctly to run periodically.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str, cron_expression: str) -> httpx.Response:
-        return httpx.post(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.post(
+            ctx,
             "{spark_url}/scheduled".format(spark_url=spark_url(product_name, ctx.obj.gateway_api_url)),
             json={
-                "cron_expression": cron_expression,
+                "cron_expression": schedule,
             },
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name, schedule)
+    r = _request(ctx)
     process_response(r)
 
 
@@ -251,19 +231,21 @@ def add(
     product_name: str,
     secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
 ):
+    """Add a set of secrets for a spark job.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str, payload: typing.Dict) -> httpx.Response:
-        return httpx.post(
+    def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
+        return util.post(
+            ctx,
             secret_url(product_name, ctx.obj.gateway_api_url),
             json=payload,
-            headers=bearer(ctx),
         )
     payload = {"data": {}}
     for s in secrets:
         name, value = s.split(":")
         payload["data"][name] = value
 
-    r = _request(ctx, product_name, payload)
+    r = _request(ctx, payload)
 
     process_response(r)
 
@@ -274,19 +256,24 @@ def update(
     product_name: str,
     secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
 ):
+    """Update existing secrets.
+
+    This will overwrite existing keys, and add new keys, any keys that already
+    exist but aren't provided will remain.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str, payload: typing.Dict) -> httpx.Response:
-        return httpx.patch(
+    def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
+        return util.patch(
+            ctx,
             secret_url(product_name, ctx.obj.gateway_api_url),
             json=payload,
-            headers=bearer(ctx),
         )
     payload = {"data": {}}
     for s in secrets:
         name, value = s.split(":")
         payload["data"][name] = value
 
-    r = _request(ctx, product_name, payload)
+    r = _request(ctx, payload)
 
     process_response(r)
 
@@ -296,14 +283,16 @@ def remove(
     ctx: typer.Context,
     product_name: str,
 ):
+    """Remove secret.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.delete(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.delete(
+            ctx,
             secret_url(product_name, ctx.obj.gateway_api_url),
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name)
+    r = _request(ctx)
 
     process_response(r)
 
@@ -314,16 +303,17 @@ def remove_key(
     product_name: str,
     keys: typing.List[str] = typer.Option(..., "--key", "-k", help="Key name you wish to remove from secret"),
 ):
+    """Remove a set of keys from a secret.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str, keys: typing.List[str]) -> httpx.Response:
-        return httpx.request(
-            "DELETE",
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.delete(
+            ctx,
             "{secret_url}/key".format(secret_url=secret_url(product_name, ctx.obj.gateway_api_url)),
             json={"keys": keys},
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name, keys)
+    r = _request(ctx)
 
     process_response(r)
 
@@ -333,13 +323,15 @@ def get(
     ctx: typer.Context,
     product_name: str,
 ):
+    """Get existing secret keys.
+    """
     @ensure_login
-    def _request(ctx: typer.Context, product_name: str) -> httpx.Response:
-        return httpx.get(
+    def _request(ctx: typer.Context) -> httpx.Response:
+        return util.get(
+            ctx,
             secret_url(product_name, ctx.obj.gateway_api_url),
-            headers=bearer(ctx),
         )
 
-    r = _request(ctx, product_name)
+    r = _request(ctx)
 
     process_response(r)
