@@ -62,17 +62,14 @@ def get_schema_profile(profile_name, allow_missing: bool = False, **kwargs):
         return schema.Profile(**kwargs)
     except pydantic.ValidationError as e:
         required_fields = [err["loc"][0] for err in e.errors() if err["msg"] == "field required"]
-        if required_fields:
-            field_names = ", ".join(required_fields)
-            send_output(
-                msg=(
-                    "Profile dotfile doesn't include fields: {}. "
-                    "Use neosctl -p {} profile init"
-                ).format(field_names, profile_name),
-                exit_code=1,
-            )
-
-        raise
+        field_names = ", ".join(required_fields)
+        send_output(
+            msg=(
+                "Profile dotfile doesn't include fields: {}. "
+                "Use neosctl -p {} profile init"
+            ).format(field_names, profile_name),
+            exit_code=1,
+        )
 
 
 def get_user_profile_section(c: configparser.ConfigParser, profile_name: str, allow_missing: bool = False):
@@ -136,22 +133,25 @@ def remove_config(
     return ctx.obj.config
 
 
-def _get_index_params(
-    page: int,
-    page_size: int,
-    filters: typing.Optional[str],
-    sort: typing.Optional[str],
-    columns: typing.Optional[str],
-) -> typing.Dict:
-    params = {}
+def _request(ctx: typer.Context, method: str, url: str, **kwargs):
+    return httpx.request(method=method, url=url, headers=bearer(ctx), verify=not ctx.obj.profile.ignore_tls, **kwargs)
 
-    params["page"] = page
-    params["page_size"] = page_size
-    if filters:
-        params["filters"] = filters
-    if sort:
-        params["sort"] = sort
-    if columns:
-        params["columns"] = columns
 
-    return params
+def get(ctx: typer.Context, url: str, **kwargs):
+    return _request(ctx, "GET", url, **kwargs)
+
+
+def post(ctx: typer.Context, url: str, **kwargs):
+    return _request(ctx, "POST", url, **kwargs)
+
+
+def put(ctx: typer.Context, url: str, **kwargs):
+    return _request(ctx, "PUT", url, **kwargs)
+
+
+def patch(ctx: typer.Context, url: str, **kwargs):
+    return _request(ctx, "PATCH", url, **kwargs)
+
+
+def delete(ctx: typer.Context, url: str, **kwargs):
+    return _request(ctx, "DELETE", url, **kwargs)
